@@ -12,8 +12,9 @@
 class IFileServFactory : public IPlugin
 {
 public:
-	virtual IFileServ * createFileServ(unsigned short tcpport, unsigned short udpport, const std::string &name="", bool use_fqdn_default=false, bool enable_background_priority=true)=0;
-	virtual IFileServ * createFileServNoBind(const std::string &name="", bool use_fqdn_default=false)=0;
+	virtual bool optainBackupPrivileges() = 0;
+	virtual IFileServ * createFileServ(unsigned short tcpport, unsigned short udpport, const std::string &name="", bool use_fqdn_default=false, bool enable_background_priority=true, bool enable_backup_semantics = true)=0;
+	virtual IFileServ * createFileServNoBind(const std::string &name="", bool use_fqdn_default=false, bool enable_background_priority = true, bool enable_backup_semantics = true)=0;
 	virtual void destroyFileServ(IFileServ *filesrv)=0;
 	virtual void setPermissionCallback(IPermissionCallback* new_permission_callback)=0;
 	virtual std::string getDefaultServerName(bool use_fqdn)=0;
