@@ -518,70 +518,7 @@ std::string replaceonce(std::string tor, std::string tin, std::string data)
         return data;
 }
 
-//--------------------------------------------------------------------
-/**
-	Tokenizes a string "str" and stores the tokens in "tokens". "seps"
-	is a string or character which defines the seperator (the breaks
-	between the tokens) - strings, which are declared by two \" are not
-	tokenized = treated as a single token
-*/
-void	Tokenize(const std::string& str, std::vector<std::string> &tokens, std::string seps)
-{
-	// one-space line for storing blank lines in the file
-	std::string blankLine = " ";
-
-	// pos0 and pos1 store the scope of the current turn, i stores
-	// the position of the symbol \".
-	s32 pos0 = 0, pos1 = 0, i = 0;
-	while(true)
-	{ 
-		// find the next seperator
-		pos1 = (s32)str.find_first_of(seps, pos0);
-		// find the next \" 
-		i    = (s32)str.find_first_of("\"", pos0);
-	    
-		// if the end is reached..
-	    if(pos1 == std::string::npos)
-	    {
-			// ..push back the string to the end
-			tokens.push_back(str.substr(pos0, str.size()));
-			break;
-	    }  
-		// if a string \" was found before the next seperator...
-	    if(( i<pos1 )&&( i > 0 ))
-	    {      
-			// .. find the end of the string and push it back, strings
-			// are treated like a single token
-			pos1 = (s32)str.find_first_of("\"", i+1);
-			tokens.push_back(str.substr(pos0, pos1-pos0+1));
-	    }  
-		// if two seperators are found in a row, the file has a blank
-		// line, in this case the one-space string is pushed as a token
-		else if( pos1==pos0 )
-		{
-			tokens.push_back(blankLine);
-		}
-	    else
-            // if no match is found, we have a simple token with the range
-			// stored in pos0/1
-			tokens.push_back(str.substr(pos0, (pos1 - pos0)));
-
-		// equalize pos
-		pos0=pos1;
-		// increase 
-	    ++pos1;
-		// added for ini-file!
-		// increase by length of seps
-		++pos0;
-	}
-	  
-	// loop through all tokens and check for empty tokens which may result
-	// as garbage through the process
-	for(s32 i=0;i<(s32)tokens.size();i++)
-		if(tokens[i]=="") tokens.erase(tokens.begin()+(i--));
-}
-
-void TokenizeMail(const std::string& str, std::vector<std::string> &tokens, std::string seps)
+void Tokenize(const std::string& str, std::vector<std::string> &tokens, std::string seps)
 {
 	// one-space line for storing blank lines in the file
 	std::string blankLine = "";
@@ -741,12 +678,57 @@ std::string EscapeParamString(const std::string &pStr)
 		case ' ': ret+="%20"; break;
 		case '#': ret+="%23"; break;
 		case '+': ret+="%2B"; break;
+		case '\n': ret += "%0A"; break;
+		case '\r': ret += "%0D"; break;
 		default: ret+=pStr[i]; break;
 		}
 	}
 	return ret;
 }
 
+std::string EscapePathParamString(const std::string & pStr)
+{
+	std::string ret;
+	ret.reserve(pStr.size());
+	for (size_t i = 0; i<pStr.size(); ++i)
+	{
+		switch (pStr[i])
+		{
+		case '%': ret += "%25"; break;
+		case '/': ret += "%2F"; break;
+		case '|': ret += "%7C"; break;
+		case ';': ret += "%3B"; break;
+		default: ret += pStr[i]; break;
+		}
+	}
+	return ret;
+}
+
+std::string UnescapeParamString(const std::string & str)
+{
+	char xc = '%';
+	std::string ret;
+	ret.reserve(str.size());
+	for (size_t i = 0; i<str.size(); i++)
+	{
+		if (str[i] == xc && i + 2<str.size())
+		{
+			std::string data; data.push_back(str[i + 1]); data.push_back(str[i + 2]);
+			unsigned char ch = (unsigned char)hexToULong(data);
+			if (ch != 0)
+			{
+				ret += ch;
+			}
+			i += 2;
+		}
+		else
+		{
+			ret += str[i];
+		}
+	}
+	return ret;
+}
+
 void EscapeCh(std::string &pStr, char ch)
 {
 	std::string ins;
@@ -792,7 +774,7 @@ void ParseParamStrHttp(const std::string &pStr, std::map<std::string,std::string
 		{
 			pos=1;
 		}
-		else if( (ch=='&'||ch=='$') && pos==1 )
+		else if( ch=='&' && pos==1 )
 		{
 			pos=0;
 			std::string wv=htmldecode(value, false);
@@ -1259,7 +1241,7 @@ std::string UnescapeHTML(const std::string &html)
 	ret=greplace("&gt;", ">", ret);
 	ret=greplace("&quot;", "\"", ret);
 	ret=greplace("&#x27;", "'", ret);
-	ret = greplace("&#x2F", "/", ret);
+	ret = greplace("&#x2F;", "/", ret);
 	return ret;
 }
 
@@ -1274,7 +1256,7 @@ std::string EscapeHTML(const std::string & html)
 		else if (html[i] == '&') ret += "&amp;";
 		else if (html[i] == '\"') ret += "&quot;";
 		else if (html[i] == '\'') ret += "&#x27;";
-		else if (html[i] == '/') ret += "&#x2F";
+		else if (html[i] == '/') ret += "&#x2F;";
 		else ret += html[i];
 	}
 	return ret;
