@@ -38,7 +38,6 @@ std::string convert(unsigned int i);
 std::string findextension(const std::string& pString);
 std::string replaceonce(std::string tor, std::string tin, std::string data);
 void Tokenize(const std::string& str, std::vector<std::string> &tokens, std::string seps);
-void TokenizeMail(const std::string& str, std::vector<std::string> &tokens, std::string seps);
 bool str_isnumber(char ch);
 bool isletter(char ch);
 void strupper(std::string *pStr);
@@ -81,6 +80,8 @@ std::string PrettyPrintBytes(_i64 bytes);
 std::string PrettyPrintSpeed(size_t bps);
 std::string PrettyPrintTime(int64 ms);
 std::string EscapeParamString(const std::string &pStr);
+std::string EscapePathParamString(const std::string &pStr);
+std::string UnescapeParamString(const std::string& str);
 
 namespace
 {
@@ -216,6 +217,20 @@ float little_endian(float x)
 	}
 }
 
+double little_endian(double x)
+{
+	if (is_big_endian())
+	{
+		uint64* ptr = reinterpret_cast<uint64*>(&x);
+		uint64 ret = endian_swap(*ptr);
+		return *reinterpret_cast<double*>(&ret);
+	}
+	else
+	{
+		return x;
+	}
+}
+
 unsigned int big_endian(unsigned int x)
 {
 	if(!is_big_endian())
