@@ -119,7 +119,12 @@ _u32 FileClientChunked::GetFilePatch(std::string remotefn, IFile *orig_file, IFi
 	extent_iterator.reset();
 	curr_sparse_extent.offset = -1;
 
-	return GetFile(remotefn, predicted_filesize, file_id, sparse_extents_f);
+	_u32 rc = GetFile(remotefn, predicted_filesize, file_id, sparse_extents_f);
+
+	if (has_error)
+		return ERR_ERROR;
+
+	return rc;
 }
 
 _u32 FileClientChunked::GetFileChunked(std::string remotefn, IFile *file, IFile *chunkhashes, IFsFile *hashoutput, _i64& predicted_filesize, int64 file_id, bool is_script, IFile** sparse_extents_f)
@@ -137,7 +142,12 @@ _u32 FileClientChunked::GetFileChunked(std::string remotefn, IFile *file, IFile
 	extent_iterator.reset();
 	curr_sparse_extent.offset = -1;
 	
-	return GetFile(remotefn, predicted_filesize, file_id, sparse_extents_f);
+	_u32 rc = GetFile(remotefn, predicted_filesize, file_id, sparse_extents_f);
+
+	if (has_error)
+		return ERR_ERROR;
+
+	return rc;
 }
 
 _u32 FileClientChunked::GetFile(std::string remotefn, _i64& filesize_out, int64 file_id, IFile** sparse_extents_f)
@@ -153,7 +163,12 @@ _u32 FileClientChunked::GetFile(std::string remotefn, _i64& filesize_out, int64
 		assert(next->remote_filename==remotefn);
 		assert(next->curr_file_id == file_id);
 
-		return next->GetFile(remotefn, filesize_out, file_id, sparse_extents_f);
+		_u32 rc = next->GetFile(remotefn, filesize_out, file_id, sparse_extents_f);
+
+		if (next->hasError())
+			return ERR_ERROR;
+
+		return rc;
 	}
 	else if(parent!=NULL && !queue_only)
 	{
@@ -546,20 +561,45 @@ _u32 FileClientChunked::GetFile(std::string remotefn, _i64& filesize_out, int64
 			
 		}
 
+		_u32 flush_rc = ERR_SUCCESS;
 		if(needs_flush)
 		{
-			Flush(getPipe());
+			flush_rc = Flush(getPipe());
 			needs_flush=false;
 		}
 
 		if(queue_only)
 		{
-			return ERR_SUCCESS;
+			return flush_rc;
+		}
+
+		if (remote_filesize > 0 &&
+			!initial_read &&
+			next_chunk >= num_total_chunks
+			&& pending_chunks.empty()
+			&& state == CS_ID_FIRST)
+		{
+			_u32 err = ERR_SUCCESS;
+			if (curr_is_script)
+			{
+				err = freeFile();
+			}
+
+			if (err == ERR_SUCCESS)
+			{
+				Server->Log("Successful (2). Returning filesize " + convert(remote_filesize), LL_DEBUG);
+				filesize_out = remote_filesize;
+			}
+			return err;
 		}
 
 		size_t rc;
 		char* buf;
-		if(initial_read && !initial_bytes.empty())
+		if (flush_rc != ERR_SUCCESS)
+		{
+			rc = 0;
+		}
+		else if(initial_read && !initial_bytes.empty())
 		{
 			rc = initial_bytes.size();
 			buf = &initial_bytes[0];
@@ -568,6 +608,13 @@ _u32 FileClientChunked::GetFile(std::string remotefn, _i64& filesize_out, int64
 		{
 			buf = stack_buf;
 			rc = getPipe()->Read(buf, BUFFERSIZE, 0);
+
+			if (initial_read && rc > 0 && rc < 10 && buf[0] == ID_ERR)
+			{
+				Server->Log("Received ID_ERR from server fc_chunked rc=" + convert(rc) + ". Reconnecting...", LL_WARNING);
+				rc = 0;
+				flush_rc = ERR_ERROR;
+			}
 		}
 
 		initial_read = false;
@@ -575,7 +622,7 @@ _u32 FileClientChunked::GetFile(std::string remotefn, _i64& filesize_out, int64
 		
 		if(rc==0)
 		{
-			if(getPipe()->hasError())
+			if(flush_rc!=ERR_SUCCESS || getPipe()->hasError())
 			{
 				Server->Log("Pipe has error. Reconnecting...", LL_DEBUG);
 				if(!Reconnect(true))
@@ -607,7 +654,7 @@ _u32 FileClientChunked::GetFile(std::string remotefn, _i64& filesize_out, int64
 					
 					if (err == ERR_SUCCESS)
 					{
-						Server->Log("Successfull. Returning filesize " + convert(remote_filesize), LL_DEBUG);
+						Server->Log("Successful. Returning filesize " + convert(remote_filesize), LL_DEBUG);
 						filesize_out = remote_filesize;
 					}
 				}
@@ -687,7 +734,8 @@ _u32 FileClientChunked::handle_data( char* buf, size_t bsize, bool ignore_filesi
 			if ((remote_filesize != -1 &&
 				remote_filesize > 0 &&
 				next_chunk >= num_total_chunks
-				&& pending_chunks.empty())
+				&& pending_chunks.empty()
+				&& state==CS_ID_FIRST)
 				|| getfile_done)
 			{
 
@@ -1430,6 +1478,8 @@ void FileClientChunked::writeFileRepeat(IFile *f, const char *buf, size_t bsize)
 	if(rc==0)
 	{
 		Server->Log("Fatal error writing to file in writeFileRepeat. Write error in Chunked File transfer. "+os_last_error_str(), LL_ERROR);
+		has_error = true;
+		getPipe()->shutdown();
 	}
 }
 
@@ -1556,7 +1606,21 @@ void FileClientChunked::writePatch(_i64 pos, unsigned int length, char *buf, boo
 			}
 			else
 			{
-				writePatchInt(pos, length, buf);
+				if (!last && length % c_chunk_size != 0)
+				{
+					const unsigned int wchunks = length / c_chunk_size;
+					const unsigned int towrite = wchunks * c_chunk_size;
+					writePatchInt(pos, towrite, buf);
+
+					const unsigned int wleft = length - towrite;
+					memcpy(&patch_buf[patch_buf_pos], buf + towrite, wleft);
+					patch_buf_start = pos + towrite;
+					patch_buf_pos += wleft;
+				}
+				else
+				{
+					writePatchInt(pos, length, buf);
+				}
 			}
 		}
 	}
@@ -1645,6 +1709,9 @@ _i64 FileClientChunked::getTransferredBytes(void)
 
 bool FileClientChunked::Reconnect(bool rerequest)
 {
+	if (has_error)
+		return false;
+
 	if(queue_callback!=NULL)
 	{
 		queue_callback->resetQueueChunked();
@@ -2263,7 +2330,7 @@ void FileClientChunked::adjustOutputFilesizeOnFailure( _i64& filesize_out )
 				_u32 read;
 				do 
 				{
-					bool has_error;
+					bool has_error = false;
 					read = m_chunkhashes->Read(&buffer[0], 4096, &has_error);
 
 					if(has_error)
@@ -2297,7 +2364,7 @@ void FileClientChunked::adjustOutputFilesizeOnFailure( _i64& filesize_out )
 	}
 	
 
-	Server->Log("Not successfull. Returning filesize "+convert(filesize_out), LL_DEBUG);
+	Server->Log("Not successful. Returning filesize "+convert(filesize_out), LL_DEBUG);
 }
 
 _u32 FileClientChunked::Flush(IPipe* fpipe)
@@ -2409,3 +2476,4 @@ _i64 FileClientChunked::getRealTransferredBytes()
 	}
 	return tbytes;
 }
+
