@@ -34,6 +34,9 @@ const uchar ID_INFORM_METADATA_STREAM_END=11;
 const uchar ID_FLUSH_SOCKET=13;
 const uchar ID_SCRIPT_FINISH = 14;
 const uchar ID_FREE_SERVER_FILE=18;
+const uchar ID_STOP_PHASH = 19;
+// Server returns ERR to FILESRV request
+const uchar ID_ERR = 3;
 
 //errors
 const unsigned int ERR_SEEKING_FAILED = 0;
