@@ -51,7 +51,8 @@ protected:
 
 	bool doImage(const std::string &pLetter, const std::string &pParentvhd, int incremental, int incremental_ref,
 		bool transfer_checksum, std::string image_file_format, bool transfer_bitmap, bool transfer_prev_cbitmap);
-	unsigned int writeMBR(ServerVHDWriter* vhdfile, uint64 volsize);
+	unsigned int writeMBR(ServerVHDWriter* vhdfile, uint64 volsize, bool gpt_protective);
+	bool writeGPT(ServerVHDWriter* vhdfile, uint64 volsize, unsigned int mbr_offset);
 	int64 updateNextblock(int64 nextblock, int64 currblock, sha256_ctx* shactx, unsigned char* zeroblockdata,
 		bool parent_fn, IFile* hashfile, IFile* parenthashfile, unsigned int blocksize,
 		int64 mbr_offset, int64 vhd_blocksize, bool &warned_about_parenthashfile_error, int64 empty_vhdblock_start,
