@@ -25,7 +25,7 @@
 #include "server_dir_links.h"
 #include "server_running.h"
 #include "server_cleanup.h"
-#include "ServerDownloadThread.h"
+#include "ServerDownloadThreadGroup.h"
 #include "FileIndex.h"
 #include <stack>
 #include "../urbackupcommon/file_metadata.h"
@@ -173,7 +173,7 @@ bool IncrFileBackup::doFileBackup()
 	std::auto_ptr<FileClientChunked> fc_chunked;
 	if(intra_file_diffs)
 	{
-		if(client_main->getClientChunkedFilesrvConnection(fc_chunked, server_settings.get(), this, 10000))
+		if(client_main->getClientChunkedFilesrvConnection(fc_chunked, server_settings.get(), this, 60000))
 		{
 			fc_chunked->setProgressLogCallback(this);
 			fc_chunked->setDestroyPipe(true);
@@ -193,7 +193,7 @@ bool IncrFileBackup::doFileBackup()
 			return false;
 		}
 	}
-	_u32 rc=client_main->getClientFilesrvConnection(&fc, server_settings.get(), 10000);
+	_u32 rc=client_main->getClientFilesrvConnection(&fc, server_settings.get(), 60000);
 	if(rc!=ERR_CONNECTED)
 	{
 		ServerLogger::Log(logid, "Incremental Backup of "+clientname+" failed - CONNECT error -2", LL_ERROR);
@@ -323,15 +323,18 @@ bool IncrFileBackup::doFileBackup()
 		}
 
 		ServerLogger::Log(logid, clientname+": Creating snapshot...", LL_INFO);
+
+		std::string snap_startup_del = zfs_file ? "" : ".startup-del";
 		std::string errmsg;
-		if(!SnapshotHelper::snapshotFileSystem(false, clientname, last.path, backuppath_single, errmsg)
-			|| !SnapshotHelper::isSubvolume(false, clientname, backuppath_single) )
+		if(!SnapshotHelper::snapshotFileSystem(false, clientname, last.path, backuppath_single+ snap_startup_del, errmsg)
+			|| !SnapshotHelper::isSubvolume(false, clientname, backuppath_single+ snap_startup_del) )
 		{
 			errmsg = trim(errmsg);
 			ServerLogger::Log(logid, "Creating new snapshot failed (Server error) "
 				+(errmsg.empty()?os_last_error_str(): ("\""+errmsg+"\"")), LL_WARNING);
 
-			if (SnapshotHelper::isSubvolume(false, clientname, backuppath_single))
+			if (SnapshotHelper::isSubvolume(false, clientname, backuppath_single+".startup-del")
+				|| SnapshotHelper::isSubvolume(false, clientname, backuppath_single) )
 			{
 				if (zfs_file)
 				{
@@ -367,16 +370,16 @@ bool IncrFileBackup::doFileBackup()
 					return false;
 				}
 
-				if (!os_link_symbolic(mountpoint, backuppath + "_new"))
+				if (!os_link_symbolic(mountpoint, backuppath + ".startup-del"))
 				{
 					ServerLogger::Log(logid, "Could create symlink to mountpoint at " + backuppath + " to " + mountpoint + ". " + os_last_error_str(), LL_ERROR);
 					has_early_error = true;
 					return false;
 				}
 
-				if (!os_rename_file(backuppath + "_new", backuppath))
+				if (!os_rename_file(backuppath + ".startup-del", backuppath))
 				{
-					ServerLogger::Log(logid, "Could rename symlink at " + backuppath + "_new to " + backuppath + ". " + os_last_error_str(), LL_ERROR);
+					ServerLogger::Log(logid, "Could rename symlink at " + backuppath + ".startup-del to " + backuppath + ". " + os_last_error_str(), LL_ERROR);
 					has_early_error = true;
 					return false;
 				}
@@ -403,29 +406,50 @@ bool IncrFileBackup::doFileBackup()
 					return false;
 				}
 
-				if (!os_link_symbolic(mountpoint, backuppath + "_new"))
+				if (!os_link_symbolic(mountpoint, backuppath + ".startup-del"))
 				{
 					ServerLogger::Log(logid, "Could create symlink to mountpoint at " + backuppath + " to " + mountpoint + ". " + os_last_error_str(), LL_ERROR);
 					has_early_error = true;
 					return false;
 				}
 
-				if (!os_rename_file(backuppath + "_new", backuppath))
+				Server->deleteFile(os_file_prefix(backuppath + ".startup-del" + os_file_sep() + ".hashes" + os_file_sep() + sync_fn));
+				os_sync(backuppath + ".startup-del" + os_file_sep() + ".hashes");
+
+				if (!os_rename_file(backuppath + ".startup-del", backuppath))
 				{
-					ServerLogger::Log(logid, "Could rename symlink at " + backuppath + "_new to " + backuppath + ". " + os_last_error_str(), LL_ERROR);
+					ServerLogger::Log(logid, "Could rename symlink at " + backuppath + ".startup-del to " + backuppath + ". " + os_last_error_str(), LL_ERROR);
+					has_early_error = true;
+					return false;
+				}
+
+				if (FileExists(backuppath_hashes + os_file_sep() + sync_fn))
+				{
+					ServerLogger::Log(logid, "Could not delete sync file. File still exists.", LL_ERROR);
 					has_early_error = true;
 					return false;
 				}
 			}
-
-			Server->deleteFile(os_file_prefix(backuppath_hashes + os_file_sep() + sync_fn));
-			os_sync(backuppath_hashes);
-
-			if (FileExists(backuppath_hashes + os_file_sep() + sync_fn))
+			else
 			{
-				ServerLogger::Log(logid, "Could not delete sync file. File still exists.", LL_ERROR);
-				has_early_error = true;
-				return false;
+				Server->deleteFile(os_file_prefix(backuppath + ".startup-del" + os_file_sep() + ".hashes" + os_file_sep() + sync_fn));
+				os_sync(backuppath + ".startup-del" + os_file_sep() + ".hashes");
+
+				if (FileExists(backuppath_hashes + ".startup-del" + os_file_sep() + sync_fn))
+				{
+					ServerLogger::Log(logid, "Could not delete sync file. File still exists.", LL_ERROR);
+					has_early_error = true;
+					return false;
+				}
+
+				if (!os_rename_file(backuppath + ".startup-del",
+					backuppath))
+				{
+					ServerLogger::Log(logid, "Error renaming new backup subvolume from " + backuppath + ".startup-del to " + backuppath + ". "
+						+ os_last_error_str(), LL_ERROR);
+					has_early_error = true;
+					return false;
+				}
 			}
 		}
 	}
@@ -501,19 +525,17 @@ bool IncrFileBackup::doFileBackup()
 	bool backup_with_components;
 	_i64 files_size = getIncrementalSize(tmp_filelist, diffs, backup_with_components);
 
-	std::auto_ptr<ServerDownloadThread> server_download(new ServerDownloadThread(fc, fc_chunked.get(), backuppath,
+	std::auto_ptr<ServerDownloadThreadGroup> server_download(new ServerDownloadThreadGroup(fc, fc_chunked.get(), backuppath,
 		backuppath_hashes, last_backuppath, last_backuppath_complete,
 		hashed_transfer, intra_file_diffs, clientid, clientname, clientsubname,
 		use_tmpfiles, tmpfile_path, server_token, use_reflink,
 		backupid, r_incremental, hashpipe_prepare, client_main, client_main->getProtocolVersions().filesrv_protocol_version,
 		incremental_num, logid, with_hashes, shares_without_snapshot, with_sparse_hashing, metadata_download_thread.get(),
-		backup_with_components, filepath_corrections, max_file_id));
+		backup_with_components, server_settings->getSettings()->download_threads, server_settings.get(), 
+		intra_file_diffs, filepath_corrections, max_file_id));
 
 	bool queue_downloads = client_main->getProtocolVersions().filesrv_protocol_version>2;
 
-	THREADPOOL_TICKET server_download_ticket = 
-		Server->getThreadPool()->execute(server_download.get(), "fbackup load");
-
 	char buffer[4096];
 	_u32 read;
 	std::string curr_path;
@@ -583,6 +605,13 @@ bool IncrFileBackup::doFileBackup()
 			bool b=list_parser.nextEntry(buffer[i], cf, &extra_params);
 			if(b)
 			{
+				if (folder_files.empty())
+				{
+					ServerLogger::Log(logid, "File list is corrupted (folder_files is empty)", LL_ERROR);
+					has_read_error = true;
+					break;
+				}
+
 				std::string osspecific_name;
 
 				if(!cf.isdir || cf.name!="..")
@@ -597,7 +626,7 @@ bool IncrFileBackup::doFileBackup()
 						if(cf.name=="..")
 						{
 							--skip_dir_completely;
-							if(skip_dir_completely>0)
+							if(skip_dir_completely>0 && !folder_files.empty())
 							{
 								curr_os_path=ExtractFilePath(curr_os_path, "/");
 								curr_path=ExtractFilePath(curr_path, "/");
@@ -830,7 +859,7 @@ bool IncrFileBackup::doFileBackup()
 											break;
 										}
 
-										metadata_srcpath = last_backuppath_hashes + convertToOSPathFromFileClient(orig_curr_os_path + "/" + escape_metadata_fn(cf.name));
+										metadata_srcpath = last_backuppath_hashes + convertToOSPathFromFileClient(orig_curr_os_path + "/" + escape_metadata_fn(osspecific_name));
 									}
 									else
 									{
@@ -850,7 +879,7 @@ bool IncrFileBackup::doFileBackup()
 								}
 								else
 								{
-									metadata_srcpath = last_backuppath_hashes + convertToOSPathFromFileClient(orig_curr_os_path + "/" + escape_metadata_fn(cf.name));
+									metadata_srcpath = last_backuppath_hashes + convertToOSPathFromFileClient(orig_curr_os_path + "/" + escape_metadata_fn(osspecific_name));
 								}
 
 								if(!createSymlink(backuppath+local_curr_os_path, depth, sym_target->second, orig_sep, true))
@@ -860,7 +889,7 @@ bool IncrFileBackup::doFileBackup()
 									break;
 								}					
 
-								metadata_fn = backuppath_hashes + convertToOSPathFromFileClient(orig_curr_os_path + "/" + escape_metadata_fn(cf.name)); 
+								metadata_fn = backuppath_hashes + convertToOSPathFromFileClient(orig_curr_os_path + "/" + escape_metadata_fn(osspecific_name));
 								
 								symlinked_file=true;
 							}
@@ -1010,7 +1039,7 @@ bool IncrFileBackup::doFileBackup()
 							}							
 						}
 					}
-					else //cf.name==".."
+					else if(!folder_files.empty()) //cf.name==".."
 					{
 						if((indirchange || dir_diff_stack.top()) && client_main->getProtocolVersions().file_meta>0 && !script_dir)
 						{
@@ -1098,7 +1127,8 @@ bool IncrFileBackup::doFileBackup()
 							&& extra_params.find("sym_target")==extra_params.end()
 							&& extra_params.find("special") == extra_params.end()
 							&& !phash_load_offline
-							&& extra_params.find("no_hash")==extra_params.end())
+							&& extra_params.find("no_hash")==extra_params.end()
+							&& cf.size >= link_file_min_size)
 						{
 							if (!phash_load->getHash(line, curr_sha2))
 							{
@@ -1348,7 +1378,7 @@ bool IncrFileBackup::doFileBackup()
 
 	ServerLogger::Log(logid, "Waiting for file transfers...", LL_INFO);
 
-	while(!Server->getThreadPool()->waitFor(server_download_ticket, 1000))
+	while(!server_download->join(1000))
 	{
 		if(files_size==0)
 		{
@@ -1409,9 +1439,15 @@ bool IncrFileBackup::doFileBackup()
 
 	waitForFileThreads();
 
-	if( bsh->hasError() || bsh_prepare->hasError() )
+	for (size_t i = 0; i < bsh.size(); ++i)
 	{
-		disk_error=true;
+		if (bsh[i]->hasError())
+			disk_error = true;
+	}
+	for (size_t i = 0; i < bsh_prepare.size(); ++i)
+	{
+		if (bsh_prepare[i]->hasError())
+			disk_error = true;
 	}
 
 	if (!r_offline && !c_has_error && !disk_error)
@@ -1452,11 +1488,11 @@ bool IncrFileBackup::doFileBackup()
 		tmp_filelist->Seek(0);
 		line = 0;
 		size_t output_offset=0;
-		std::stack<size_t> last_modified_offsets;
 		list_parser.reset();
 		script_dir=false;
 		indirchange=false;
 		has_read_error = false;
+		depth = 0;
 		while( (read=tmp_filelist->Read(buffer, 4096, &has_read_error))>0 )
 		{
 			if (has_read_error)
@@ -1484,7 +1520,7 @@ bool IncrFileBackup::doFileBackup()
 
 						if(cf.name!="..")
 						{
-							if(cf.name=="urbackup_backup_scripts")
+							if(depth == 0 && cf.name=="urbackup_backup_scripts")
 							{
 								script_dir=true;
 							}
@@ -1516,7 +1552,8 @@ bool IncrFileBackup::doFileBackup()
 								indirchange=false;
 							}
 
-							script_dir=false;
+							if(depth==0)
+								script_dir=false;
 						}
 
 
@@ -1901,7 +1938,7 @@ bool IncrFileBackup::deleteFilesInSnapshot(const std::string clientlist_fn, cons
 		{
 			if(list_parser.nextEntry(buffer[i], curr_file, NULL))
 			{
-				if(curr_file.isdir && curr_file.name=="..")
+				if(curr_file.isdir && curr_file.name==".." && !folder_files.empty())
 				{
 					folder_files.pop();
 					curr_path=ExtractFilePath(curr_path, os_file_sep());
