@@ -20,7 +20,11 @@
 #include "dao/ServerCleanupDao.h"
 #include "../stringtools.h"
 #include "../urbackupcommon/os_functions.h"
+#ifdef NO_EMBEDDED_LMDB
+#include <lmdb.h>
+#else
 #include "lmdb/lmdb.h"
+#endif
 #include "../Interface/Server.h"
 #include "../Interface/Mutex.h"
 #include "database.h"
@@ -28,10 +32,6 @@
 #include "server_log.h"
 #include "server_status.h"
 
-#if defined(_WIN32) || defined(__APPLE__) || defined(__FreeBSD__)
-#define stat64 stat
-#endif
-
 #ifndef _WIN32
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -41,6 +41,10 @@
 #include <Windows.h>
 #endif
 
+#if defined(_WIN32) || defined(__APPLE__) || defined(__FreeBSD__)
+#define stat64 stat
+#endif
+
 namespace
 {
 	std::string getBackupfolder(IDatabase *db)
@@ -132,7 +136,16 @@ namespace
 			return false;
 		}
 
-		rc = mdb_env_set_mapsize(env, 1ULL*1024*1024*1024*1024); //1TB
+		uint64 envsize = 1ULL * 1024 * 1024 * 1024 * 1024; //1TB
+
+		int64 freespace = os_free_space(dst_folder);
+		if (freespace > 0
+			&& static_cast<uint64>(freespace)<envsize)
+		{
+			envsize = static_cast<uint64>(0.9*freespace);
+		}
+
+		rc = mdb_env_set_mapsize(env, envsize);
 
 		if (rc)
 		{
@@ -214,7 +227,7 @@ namespace
 	std::string remove_incomplete_folder(const std::string& path)
 	{
 		std::vector<std::string> toks;
-		TokenizeMail(path, toks, os_file_sep());
+		Tokenize(path, toks, os_file_sep());
 
 		std::string ret;
 		std::string inc_str = "_incomplete";
