@@ -86,6 +86,8 @@ SStartupStatus startup_status;
 #include "DataplanDb.h"
 #include "Alerts.h"
 #include "Mailer.h"
+#include "../urbackupcommon/settingslist.h"
+#include "../urbackupcommon/settings.h"
 
 #include <stdlib.h>
 #include "../Interface/DatabaseCursor.h"
@@ -98,6 +100,7 @@ SStartupStatus startup_status;
 #include "FileMetadataDownloadThread.h"
 #include "../urbackupcommon/chunk_hasher.h"
 #include "LogReport.h"
+#include "WebSocketConnector.h"
 
 #define MINIZ_NO_ZLIB_COMPATIBLE_NAMES
 #include "../common/miniz.h"
@@ -151,6 +154,7 @@ bool verify_hashes(std::string arg);
 void updateRights(int t_userid, std::string s_rights, IDatabase *db);
 int md5sum_check();
 int blockalign();
+void init_server_pubkey();
 
 std::string lang="en";
 std::string time_format_str="%Y-%m-%d %H:%M";
@@ -381,6 +385,8 @@ void detach_other_dbs(IDatabase* db)
 	db->Write("DETACH DATABASE links_db");
 }
 
+#include "../urbackupcommon/WebSocketPipe.h"
+
 DLLEXPORT void LoadActions(IServer* pServer)
 {
 	Server=pServer;
@@ -602,6 +608,8 @@ DLLEXPORT void LoadActions(IServer* pServer)
 		server_token=token;
 	}
 
+	init_server_pubkey();
+
 	Server->deleteFile("urbackup/shutdown_now");
 
 
@@ -649,6 +657,8 @@ DLLEXPORT void LoadActions(IServer* pServer)
 	ClientMain::init_mutex();
 	DataplanDb::init();
 	init_log_report();
+	ServerChannelThread::init_mutex();
+	Helper::init_mutex();
 
 	open_settings_database();
 	
@@ -743,7 +753,7 @@ DLLEXPORT void LoadActions(IServer* pServer)
 
 		std::string new_salt=ServerSettings::generateRandomAuthKey(20);
 
-		const size_t pbkdf2_rounds = 10000;
+		const size_t pbkdf2_rounds = default_pbkdf2_rounds;
 
 		std::string password_md5 = strlower(crypto_fak->generatePasswordHash(hexToBytes(Server->GenerateHexMD5(new_salt+set_admin_pw)),
 				new_salt, pbkdf2_rounds));
@@ -819,6 +829,7 @@ DLLEXPORT void LoadActions(IServer* pServer)
 	ADD_ACTION(restore_prepare_wait);
 	ADD_ACTION(scripts);
 	ADD_ACTION(status_check);
+	ADD_ACTION(restore_image);
 
 	if(Server->getServerParameter("allow_shutdown")=="true")
 	{
@@ -856,19 +867,43 @@ DLLEXPORT void LoadActions(IServer* pServer)
 
 	{
 		ServerSettings settings(Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER));
-		if(settings.getSettings()->internet_mode_enabled)
+		if(Server->getServerParameter("internet_mode_disabled")!="1")
 		{
 			std::string tmp=Server->getServerParameter("internet_port", "");
 			unsigned int port;
 			if(!tmp.empty())
 			{
-				port=atoi(tmp.c_str());
+				port = atoi(tmp.c_str());
 			}
 			else
 			{
-				port=settings.getSettings()->internet_server_port;
+				port = 55415;
+			}
+
+			std::auto_ptr<ISettingsReader> settings_db(Server->createDBSettingsReader(db, "settings_db.settings",
+				"SELECT value FROM settings_db.settings WHERE key=? AND clientid=0"));
+
+			std::string port_str = settings_db->getValue("internet_server_bind_port", std::string());
+			if (!port_str.empty())
+			{
+				port = watoi(port_str);
+			}
+
+			IServer::BindTarget internet_bind_target = IServer::BindTarget_All;
+
+			if (Server->getServerParameter("internet_localhost_only") == "1")
+			{
+				internet_bind_target = IServer::BindTarget_Localhost;
+			}
+
+			InternetService* internet_service = new InternetService(backup_server);
+			Server->StartCustomStreamService(internet_service, "InternetService", port, internet_bind_target);
+
+			if (Server->getServerParameter("internet_disable_websocket") != "1")
+			{
+				Server->addWebSocket(new WebSocketConnector(internet_service, "socket"));
+				Server->addWebSocket(new WebSocketConnector(internet_service, ""));
 			}
-			Server->StartCustomStreamService(new InternetService(backup_server), "InternetService", port);
 		}
 	}
 
@@ -2080,6 +2115,354 @@ bool upgrade58_59()
 	return b;
 }
 
+bool upgrade59_60()
+{
+	IDatabase *db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+
+	bool b = true;
+
+	b &= db->Write("ALTER TABLE settings_db.settings ADD value_client TEXT");
+	b &= db->Write("ALTER TABLE settings_db.settings ADD use INTEGER");
+	b &= db->Write("UPDATE settings_db.settings SET use="+convert(c_use_value));
+
+	IQuery* q_get = db->Prepare("SELECT value FROM settings_db.settings WHERE clientid=? AND key=?");
+	IQuery* q_update_use = db->Prepare("UPDATE settings_db.settings SET use=? WHERE clientid=?");
+	IQuery* q_update_use_key = db->Prepare("UPDATE settings_db.settings SET use=? WHERE clientid=? AND key=?");
+
+	db_results res_clients = db->Read("SELECT id FROM clients");
+	for (size_t i = 0; i < res_clients.size(); ++i)
+	{
+		int clientid = watoi(res_clients[i]["id"]);
+		
+		q_get->Bind(clientid);
+		q_get->Bind("overwrite");
+		db_results res = q_get->Read();
+		q_get->Reset();
+		if (res.empty() || res[0]["value"] != "true")
+		{
+			q_update_use->Bind(c_use_group);
+			q_update_use->Bind(clientid);
+			q_update_use->Write();
+			q_update_use->Reset();
+		}
+
+		q_get->Bind(clientid);
+		q_get->Bind("client_set_settings");
+		res = q_get->Read();
+		q_get->Reset();
+		if (!res.empty() && res[0]["value"] == "true")
+		{
+			std::vector<std::string> settings = getClientConfigurableSettingsList();
+			for (size_t j = 0; j < settings.size(); ++j)
+			{
+				q_update_use_key->Bind(c_use_value_client);
+				q_update_use_key->Bind(clientid);
+				q_update_use_key->Bind(settings[j]);
+				q_update_use_key->Write();
+				q_update_use_key->Reset();
+			}
+		}
+	}
+
+	IQuery* q_update_use_key_only = db->Prepare("UPDATE settings_db.settings SET use=? WHERE key=?");
+	std::vector<std::string> settings = getLocalizedSettingsList();
+	for (size_t i = 0; i < settings.size(); ++i)
+	{
+		q_update_use_key_only->Bind(c_use_value);
+		q_update_use_key_only->Bind(settings[i]);
+	}
+
+	return b;
+}
+
+std::string archiveSettingsParamStr(IDatabase* db, int clientid, std::string prefix)
+{
+	db_results res_c = db->Read("SELECT COUNT(*) AS c FROM settings_db.automatic_archival WHERE clientid=" + convert(clientid));
+	if (!res_c.empty() && res_c[0]["c"] == "0")
+		return std::string();
+
+	db_results res = db->Read("SELECT id, interval, interval_unit, length, length_unit, backup_types, clientid, archive_window, letters FROM settings_db.automatic_archival WHERE clientid=" + convert(clientid));
+
+	IQuery* q_set_uuid = db->Prepare("UPDATE settings_db.automatic_archival SET uuid=? WHERE id=?");
+
+	std::string ret;
+	for (size_t i = 0; i < res.size(); ++i)
+	{
+		std::string uuid;
+		uuid.resize(16);
+		Server->secureRandomFill(&uuid[0], uuid.size());
+
+		std::string idx = "_" + prefix + convert(i);
+		if (!ret.empty()) ret += "&";
+		ret += "every"+ idx +"=" + EscapeParamString(res[i]["interval"]);
+		ret += "&every_unit" + idx + "=" + EscapeParamString(res[i]["interval_unit"]);
+		ret += "&for" + idx + "=" + EscapeParamString(res[i]["length"]);
+		ret += "&for_unit" + idx + "=" + EscapeParamString(res[i]["length_unit"]);
+		ret += "&backup_type" + idx + "=" + EscapeParamString(ServerAutomaticArchive::getBackupType(watoi(res[i]["backup_types"])));
+		ret += "&window" + idx + "=" + EscapeParamString(res[i]["archive_window"]);
+		ret += "&letters" + idx + "=" + EscapeParamString(res[i]["letters"]);
+		ret += "&uuid"+idx+"=" + bytesToHex(uuid);
+
+		q_set_uuid->Bind(uuid.data(), uuid.size());
+		q_set_uuid->Bind(res[i]["id"]);
+		q_set_uuid->Write();
+		q_set_uuid->Reset();
+	}
+
+	return ret;
+}
+
+std::string fixArchiveMigration(IDatabase* db, const std::string& archive_str)
+{
+	str_map archive_settings;
+	ParseParamStrHttp(archive_str, &archive_settings);
+
+	IQuery* q_find = db->Prepare("SELECT length, length_unit FROM settings_db.automatic_archival WHERE uuid=?");
+
+	for (str_map::iterator it = archive_settings.begin();
+		it != archive_settings.end(); ++it)
+	{
+		if (next(it->first, 0, "backup_types_"))
+		{
+			std::string idx = getafter("backup_types_", it->first);
+
+			str_map::iterator it_for = archive_settings.find("for_" + idx);
+			str_map::iterator it_for_unit = archive_settings.find("for_unit_" + idx);
+			str_map::iterator it_uuid = archive_settings.find("uuid_" + idx);
+			if (it_uuid != archive_settings.end() &&
+				it_for != archive_settings.end() &&
+				it_for_unit != archive_settings.end() &&
+				it_for->second == it_for_unit->second)
+			{
+				std::string uuid = hexToBytes(it_uuid->second);
+				q_find->Bind(uuid.data(), uuid.size());
+				db_results res = q_find->Read();
+				q_find->Reset();
+
+				if (!res.empty())
+				{
+					archive_settings["for_" + idx] = res[0]["length"];
+					archive_settings["for_unit_" + idx] = res[0]["length_unit"];
+				}
+			}
+		}
+	}
+
+	std::string ret;
+	for (str_map::iterator it = archive_settings.begin();
+		it != archive_settings.end(); ++it)
+	{
+		if (!ret.empty())
+			ret += "&";
+
+		if (next(it->first, 0, "backup_types_"))
+		{
+			std::string idx = getafter("backup_types_", it->first);
+
+			if(archive_settings.find("backup_type_" + idx)== archive_settings.end())
+				ret += "backup_type_" + idx + "=" + EscapeParamString(it->second);
+		}
+		else if (next(it->first, 0, "archive_window_"))
+		{
+			std::string idx = getafter("archive_window_", it->first);
+
+			if (archive_settings.find("window_" + idx) == archive_settings.end())
+				ret += "window_" + idx + "=" + EscapeParamString(it->second);
+		}
+		else
+		{
+			ret += it->first + "=" + EscapeParamString(it->second);
+		}
+	}
+
+	return ret;
+}
+
+bool upgrade60_61()
+{
+	IDatabase *db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+
+	if (!db->Write("ALTER TABLE settings_db.automatic_archival ADD uuid BLOB"))
+	{
+		return false;
+	}
+
+	bool b = true;
+	IQuery* q_insert_setting = db->Prepare("INSERT INTO settings_db.settings (key, value, clientid, use) VALUES (?, ?, ?, ?)");
+
+	db_results res_groups = db->Read("SELECT id FROM settings_db.si_client_groups");
+	db_single_result res_global;
+	res_global["id"] = "0";
+	res_groups.push_back(res_global);
+
+	for (size_t i = 0; i < res_groups.size(); ++i)
+	{
+		int group_id = watoi(res_groups[i]["id"]);
+
+		q_insert_setting->Bind("archive");
+		q_insert_setting->Bind(archiveSettingsParamStr(db, group_id*-1, group_id==0 ? "d" : "g"));
+		q_insert_setting->Bind(group_id*-1);
+		q_insert_setting->Bind(c_use_value);
+		b &= q_insert_setting->Write();
+		q_insert_setting->Reset();
+	}
+
+	IQuery* q_get = db->Prepare("SELECT value FROM settings_db.settings WHERE clientid=? AND key=?");
+	db_results res_clients = db->Read("SELECT id FROM clients");
+	for (size_t i = 0; i < res_clients.size(); ++i)
+	{
+		int clientid = watoi(res_clients[i]["id"]);
+		int r_clientid = clientid;
+		int group_id = 0;
+		q_get->Bind(clientid);
+		q_get->Bind("group_id");
+		db_results res = q_get->Read();
+		q_get->Reset();
+		if (!res.empty())
+		{
+			group_id = watoi(res[0]["value"])*-1;
+		}
+
+		q_get->Bind(clientid);
+		q_get->Bind("overwrite");
+		res = q_get->Read();
+		q_get->Reset();
+		if (res.empty() || res[0]["value"] != "true")
+			r_clientid = group_id;
+
+		q_get->Bind(clientid);
+		q_get->Bind("overwrite_archive_settings");
+		res = q_get->Read();
+		q_get->Reset();
+		if (res.empty() || res[0]["value"] != "true")
+			r_clientid = group_id;
+
+		std::string archive_params;
+		int archive_use = c_use_value;
+		if ( r_clientid <= 0 )
+		{
+			archive_use = c_use_group;
+		}
+		else
+		{
+			archive_params = archiveSettingsParamStr(db, clientid, "c");
+		}
+
+		q_insert_setting->Bind("archive");
+		q_insert_setting->Bind(archive_params);
+		q_insert_setting->Bind(clientid);
+		q_insert_setting->Bind(archive_use);
+		b &= q_insert_setting->Write();
+		q_insert_setting->Reset();
+
+		q_insert_setting->Bind("archive_update");
+		q_insert_setting->Bind("1");
+		q_insert_setting->Bind(clientid);
+		q_insert_setting->Bind(0);
+		b &= q_insert_setting->Write();
+		q_insert_setting->Reset();
+	}
+
+	return b;
+}
+
+bool upgrade61_62()
+{
+	IDatabase* db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+	bool b= db->Write("ALTER TABLE clients ADD capa INTEGER DEFAULT 0");
+	b &= db->Write("UPDATE clients SET capa=0 WHERE capa IS NULL");
+	return b;
+}
+
+bool upgrade62_63()
+{
+	IDatabase* db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+	bool b = db->Write("ALTER TABLE clients ADD with_hashes INTEGER DEFAULT 0");
+	b &= db->Write("UPDATE clients SET with_hashes=0 WHERE with_hashes IS NULL");
+	return b;
+}
+
+bool upgrade63_64()
+{
+	IDatabase* db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+	bool b = db->Write("ALTER TABLE settings_db.settings ADD use_last_modified INTEGER");
+	b &= db->Write("UPDATE settings_db.settings SET use_last_modified=" + convert(Server->getTimeSeconds()));
+	b &= db->Write("UPDATE settings_db.settings SET value_client=value");
+	return b;
+}
+
+bool upgrade64_65()
+{
+	IDatabase* db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+
+	db_results res_port = db->Read("SELECT value FROM settings_db.settings WHERE key='internet_server_port' AND clientid=0");
+
+	bool b = true;
+	if (!res_port.empty()
+		&& res_port[0]["value"] != "55415")
+	{
+		b &= db->Write(std::string("INSERT INTO settings_db.settings (key, value, clientid) VALUES ('internet_server_bind_port', '")+convert(watoi(res_port[0]["value"]))+"', 0)");
+	}
+	
+	return b;
+}
+
+bool upgrade65_66()
+{
+	IDatabase* db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+
+	return db->Write(std::string("UPDATE settings_db.settings SET use=") + c_use_value_str + " WHERE key='virtual_clients_add'");
+}
+
+bool upgrade66_67()
+{
+	IDatabase* db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+
+	return db->Write(std::string("UPDATE settings_db.settings SET use=") + c_use_value_str + " WHERE key='internet_authkey'");
+}
+
+bool upgrade67_68()
+{
+	IDatabase* db = Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER);
+
+	db_results res = db->Read("SELECT rowid, value FROM settings_db.settings WHERE key='archive'");
+
+	IQuery* q_update = db->Prepare("UPDATE settings_db.settings SET value=? WHERE rowid=?");
+
+	if (!db->Write("UPDATE settings_db.automatic_archival SET uuid=unhex(hex(uuid))"))
+		return false;
+
+	bool ret = true;
+
+	for (size_t i = 0; i < res.size(); ++i)
+	{
+		std::string new_res = fixArchiveMigration(db, res[i]["value"]);
+		if (new_res != res[i]["value"])
+		{
+			q_update->Bind(new_res);
+			q_update->Bind(res[i]["rowid"]);
+			ret &= q_update->Write();
+			q_update->Reset();
+		}
+	}	
+
+	IQuery* q_insert_setting = db->Prepare("INSERT INTO settings_db.settings (key, value, clientid, use) VALUES (?, ?, ?, ?)");
+	db_results res_clients = db->Read("SELECT id FROM clients");
+	for (size_t i = 0; i < res_clients.size(); ++i)
+	{
+		const int clientid = watoi(res_clients[i]["id"]);
+
+		q_insert_setting->Bind("archive_update");
+		q_insert_setting->Bind("1");
+		q_insert_setting->Bind(clientid);
+		q_insert_setting->Bind(0);
+		ret &= q_insert_setting->Write();
+		q_insert_setting->Reset();
+	}
+
+	return true;
+}
+
 void upgrade(void)
 {
 	Server->destroyAllDatabases();
@@ -2101,7 +2484,7 @@ void upgrade(void)
 	
 	int ver=watoi(res_v[0]["tvalue"]);
 	int old_v;
-	int max_v=59;
+	int max_v=68;
 	{
 		IScopedLock lock(startup_status.mutex);
 		startup_status.target_db_version=max_v;
@@ -2125,8 +2508,15 @@ void upgrade(void)
 		&& ver>15)
 	{
 		cache_res=db->Read("PRAGMA cache_size");
-		ServerSettings server_settings(db);
-		db->Write("PRAGMA cache_size = -"+convert(server_settings.getSettings()->update_stats_cachesize));
+		ServerBackupDao backup_dao(db);
+		size_t update_stats_cachesize = 200 * 1024;
+		ServerBackupDao::CondString setting = backup_dao.getClientSetting("update_stats_cachesize", 0);
+		if (setting.exists
+			&& !setting.value.empty())
+		{
+			update_stats_cachesize = watoi64(setting.value);
+		}
+		db->Write("PRAGMA cache_size = -"+convert(update_stats_cachesize));
 	}
 	
 	IQuery *q_update=db->Prepare("UPDATE misc SET tvalue=? WHERE tkey='db_version'");
@@ -2451,6 +2841,69 @@ void upgrade(void)
 				}
 				++ver;
 				break;
+			case 59:
+				if (!upgrade59_60())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 60:
+				if (!upgrade60_61())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 61:
+				if (!upgrade61_62())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 62:
+				if (!upgrade62_63())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 63:
+				if (!upgrade63_64())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 64:
+				if (!upgrade64_65())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 65:
+				if (!upgrade65_66())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 66:
+				if (!upgrade66_67())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
+			case 67:
+				if (!upgrade67_68())
+				{
+					has_error = true;
+				}
+				++ver;
+				break;
 			default:
 				break;
 		}
