@@ -183,12 +183,13 @@ void BackupServer::operator()(void)
 	q_update_lastseen = db->Prepare("UPDATE clients SET lastseen=datetime(?, 'unixepoch') WHERE id=?", false);
 
 	FileClient fc(true, "");
+	FileClient::GetServersErrors fc_errors;
 
 	Server->wait(1000);
 
 	while(true)
 	{
-		findClients(fc);
+		findClients(fc, fc_errors);
 		startClients(fc);
 
 		if(!ServerStatus::isActive() && settings->getValue("autoshutdown", "false")=="true")
@@ -218,7 +219,7 @@ void BackupServer::operator()(void)
 	}
 }
 
-void BackupServer::findClients(FileClient &fc)
+void BackupServer::findClients(FileClient &fc, FileClient::GetServersErrors& fc_errors)
 {
 	std::vector<FileClient::SAddrHint> addr_hints;
 	if(q_get_extra_hostnames!=NULL)
@@ -246,10 +247,10 @@ void BackupServer::findClients(FileClient &fc)
 		}
 	}
 
-	_u32 rc=fc.GetServers(true, addr_hints);
+	_u32 rc=fc.GetServers(true, addr_hints, fc_errors);
 	while(rc==ERR_CONTINUE)
 	{
-		rc=fc.GetServers(false, addr_hints);
+		rc=fc.GetServers(false, addr_hints, fc_errors);
 
 		if(exitpipe->isReadable())
 		{
@@ -480,6 +481,15 @@ void BackupServer::startClients(FileClient &fc)
 				if( (it->second.changecount>5 && none_fits)
 					|| found_lan)
 				{
+					Server->Log("New client address for "+ curr_info.name+": " + curr_info.addr.toString()+" old address: "+it->second.addr.toString(), LL_INFO);
+					for (size_t j = 0; j < client_info.size(); ++j)
+					{
+						if (client_info[j].name == curr_info.name)
+						{
+							Server->Log("Available address (" + curr_info.name + "): " + client_info[j].addr.toString(), LL_INFO);
+						}
+					}
+
 					it->second.addr=curr_info.addr;
 					it->second.internet_connection=curr_info.internetclient;
 					std::string msg;
@@ -489,8 +499,6 @@ void BackupServer::startClients(FileClient &fc)
 					msg[7+sizeof(FileClient::SAddrHint)]=(curr_info.internetclient?1:0);
 					it->second.pipe->Write(msg);
 
-					Server->Log("New client address: "+it->second.addr.toString(), LL_INFO);
-
 					if(it->second.addr.is_ipv6)
 						ServerStatus::setIPv6(curr_info.name, it->second.addr.addr_ipv6);
 					else
@@ -498,6 +506,10 @@ void BackupServer::startClients(FileClient &fc)
 
 					it->second.offlinecount=0;
 				}
+				else if (!none_fits)
+				{
+					it->second.changecount = 0;
+				}
 			}
 		}
 	}
