@@ -2,6 +2,7 @@
 #include "../Interface/Mutex.h"
 
 class ClientMain;
+struct SProcess;
 
 class ServerPingThread : public IThread
 {
@@ -14,11 +15,12 @@ public:
 	bool isTimeout(void);
 
 private:
+	void setPaused(SProcess* proc, bool b);
 	ClientMain *client_main;
 	volatile bool stop;
 	volatile bool is_timeout;
 	bool with_eta;
-	const std::string& clientname;
+	std::string clientname;
 	size_t status_id;
 	std::string server_token;
 };
\ No newline at end of file
